from dynprogstorage.Wrapper_dynprogstorage import Pycplfunction, Pycplfunctionvec
import math
from numpy import random


def test_pycplfunction():
    A = Pycplfunction([-1., 1.], [-10., 0.], 0.)
    A.getBreakPoints()
    A.evalf(1)
    A.evalf(0)
    A.AddSimple(1., 2., 3., 4.)
    A.getBreakPoints()

    B = Pycplfunction([-1, 1], [-math.inf, 0], 0.)
    B.getBreakPoints()


def test_pycplfunctionvec():
    C = Pycplfunctionvec([-1., 1.], [-10., 0.], [0, 0])
    C.size()
    C.Maxf_1Breaks_withO([-1., 3.], [-12., 2.], [0, 0])
    E = Pycplfunctionvec([-2., 2.], [-12., 2.], [0, 0])
    # E = E.Max_(D, C)


def test_one_breakpoint():
    n = 100
    S1 = random.uniform(low=1, high=10, size=n).tolist()
    E = Pycplfunctionvec(S1, [-math.inf] * n, [0] * n)
    PP = 1
    CC = 5 * PP
    xEtoile = E.OptimMargInt([-PP] * n, [PP] * n, [0] * n, [CC] * n)
    # print(xEtoile)


def test_two_breakpoints():
    n = 100
    S1 = random.uniform(low=1, high=10, size=n)
    S2 = S1 + 2
    E = Pycplfunctionvec(S1.tolist(), [-math.inf] * n, [0] * n,  # S1, B1, f0
                                        S2.tolist(), [0] * n)  # S2, B2
    PP = 1
    CC = 5 * PP
    xEtoile = E.OptimMargInt([-PP] * n, [PP] * n, [0] * n, [CC] * n)



from dynprogstorage.wrappers import GenCostFunctionFromMarketPrices
from dynprogstorage.Wrapper_dynprogstorage import Pycplfunctionvec
from numpy import random
import time

def test_dynprog():
    ### exemples simples d'utilisation de l'outil de programmation dynamique
    ###### storage operation example
    nbTime=250
    Prices=random.uniform(1, 1000, nbTime)
    p_max=1.
    c_max=10.*p_max

    ## x_i>0 : on stocke (on consomme du réseau)
    ## x_i<0 : on produit
    ### --> phi_i(x_i) est donc un coût Achat - vente que l'on veut minimiser
    cpl_func = GenCostFunctionFromMarketPrices(Prices.tolist())
    res = cpl_func.OptimMargInt([-p_max]*nbTime,[p_max]*nbTime,[0]*nbTime,[c_max]*nbTime)
    print(res)



def test_dyn_prog_perf():
    p_max = 1.
    c_max = 10. * p_max
    for nbTime in [1000, 10000, 100000]:
        Prices = random.uniform(1, 1000, nbTime)
        start = time.time()
        cpl_func = GenCostFunctionFromMarketPrices(Prices.tolist())
        res = cpl_func.OptimMargInt([-p_max] * nbTime, [p_max] * nbTime, [0] * nbTime, [c_max] * nbTime)
        print('Elapsed time for ' + str(nbTime) + ' steps : ' + str(time.time() - start))

def test_max():
    ### exemples simples d'utilisation de l'outil de programmation dynamique
    ###### storage operation example
    nbTime=250
    Prices1=random.uniform(1, 1000, nbTime)
    Prices2=random.uniform(1, 1000, nbTime)
    p_max=1.
    c_max=10.*p_max

    ## x_i>0 : on stocke (on consomme du réseau)
    ## x_i<0 : on produit
    ### --> phi_i(x_i) est donc un coût Achat - vente que l'on veut minimiser
    cpl_func1 = GenCostFunctionFromMarketPrices(Prices1.tolist())
    cpl_func2 = GenCostFunctionFromMarketPrices(Prices2.tolist())
    cpl_func1.vec_get(0).getBreakPoints()
    cpl_func2.vec_get(0).getBreakPoints()
    cpl_func1.vec_get(0).FirstBreakVal_
    cpl_func12=cpl_func1.Max_(cpl_func2)
    cpl_func12b=Pycplfunctionvec.MaxPycplfunctionvec(cpl_func1,cpl_func2)
    cpl_func12.vec_get(0).getBreakPoints()
    cpl_func12b.vec_get(0).getBreakPoints()

    res12 = cpl_func12.OptimMargInt([-p_max] * nbTime, [p_max] * nbTime, [0] * nbTime, [c_max] * nbTime)
    res1 = cpl_func1.OptimMargInt([-p_max] * nbTime, [p_max] * nbTime, [0] * nbTime, [c_max] * nbTime)
    res2 = cpl_func2.OptimMargInt([-p_max] * nbTime, [p_max] * nbTime, [0] * nbTime, [c_max] * nbTime)



# distutils: language = c++

from libcpp.vector cimport vector
from libcpp.map cimport map
from libcpp.string cimport string

# follow example here http://docs.cython.org/en/latest/src/userguide/wrapping_CPlusPlus.html#add-public-attributes
# Declare the class with cdef
cdef extern from "cplfunction.hpp" :
    cdef cppclass cplfunction:
        cplfunction() except +
        cplfunction(cplfunction)
        cplfunction(double,double) except +
        #cplfunction(cplfunction const) except +
        cplfunction(vector[double],vector[double],double) except +
        #cplfunction operator=(cplfunction)
        double evalf(double )
        void AddSimple(double , double , double , double )
        void print_()
        map[double,double] get_BreakPoints()
        double FirstBreakVal_
        double FirstSlopeVal_
        map[double, double] Breakpoints_



#Create Cython wrapper class http://docs.cython.org/en/latest/src/userguide/wrapping_CPlusPlus.html#create-cython-wrapper-class
# le guide cython ne fonctionne pas
# another inspiration https://stackoverflow.com/questions/33677231/how-to-expose-a-function-returning-a-c-object-to-python-without-copying-the-ob


#from cplfunction cimport cplfunction

cdef class Pycplfunction:
    cdef cplfunction thisptr ## * is required here see https://stackoverflow.com/questions/33573038/how-to-expose-a-function-returning-a-c-object-to-python-using-cython
    # typiquement le genre de bazar que l'on n'aurait pas si l'on pouvait utiliser boost-python ;)
    # would it be possible to use a switch case ?
    def __cinit__(self,vector[double] Slopes,vector[double] BreakPoints, double FirstBreakVal):
        self.thisptr = cplfunction(Slopes,BreakPoints,FirstBreakVal)

    cdef copy(self, cplfunction other):
        self.thisptr = cplfunction(other)

    def evalf(self,double x):
        return self.thisptr.evalf(x)

    def getBreakPoints(self):
        return self.thisptr.get_BreakPoints()

    def AddSimple(self,double val1,double val2,double val3,double val4):
        return self.thisptr.AddSimple(val1,val2,val3,val4)

    def Myprint(self):
        return self.thisptr.print_()

 #   def __repr__(self):
 #       return "<cplfunction: Breakpoints_={}, FirstBreakVal_={0}, FirstSlopeVal_={0}>".format(self.Breakpoints_,self.FirstBreakVal_, self.FirstSlopeVal_)

    property Breakpoints_:
        def __get__(self): return self.thisptr.Breakpoints_
        def __set__(self, map[double,double] Breakpoints_): self.thisptr.Breakpoints_ = Breakpoints_

    property FirstBreakVal_:
        def __get__(self): return self.thisptr.FirstBreakVal_
        def __set__(self, FirstBreakVal_): self.thisptr.FirstBreakVal_ = FirstBreakVal_

    property FirstSlopeVal_:
        def __get__(self): return self.thisptr.FirstSlopeVal_
        def __set__(self, FirstSlopeVal_): self.thisptr.FirstSlopeVal_ = FirstSlopeVal_

#    @property
#    def x1(self):
#        return self.c_rect.x1
#    @x1.setter
#    def x1(self, x1):
#        self.c_rect.x1 = x1

## a regarder http://nicolas-hug.com/blog/cython_notes


# distutils: language = c++
# distutils: sources = cplfunction.cpp

from libcpp.vector cimport vector
from libcpp.map cimport map
from libcpp.string cimport string


# follow example here http://docs.cython.org/en/latest/src/userguide/wrapping_CPlusPlus.html#add-public-attributes
# Declare the class with cdef
cdef extern from "cplfunction.cpp" :
    cdef cppclass cplfunctionvec:
        cplfunctionvec() except +
        cplfunctionvec(int) except +
        cplfunctionvec(cplfunctionvec &) except +
        #cplfunctionvec(cplfunctionvec const) except +
        cplfunctionvec(vector[double],vector[double],vector[double])  except +
        cplfunctionvec(vector[double],vector[double],vector[double],vector[double],vector[double])  except +
        cplfunction vec_get(int)
        int size()
        cplfunctionvec Maxf(cplfunctionvec &)
        void Max_(cplfunctionvec &,cplfunctionvec &)
        vector[cplfunction] MycplfunctionList_
        void Maxf_1Breaks_withO(vector[double],vector[double],vector[double])
        void Maxf_2Breaks_withO(vector[double],vector[double],vector[double],vector[double],vector[double])
        vector[double] EvalDeltaf2(vector[double] )
        vector[double] EvalDeltafMoins(vector[double] )
        vector[double] EvalDeltafPlus(vector[double] )
        vector[double] Evalf(vector[double])
        vector[double] OptimMargInt(vector[double],vector[double],vector[double],vector[double])


#from cplfunctionvec cimport cplfunctionvec

#https://stackoverflow.com/questions/33677231/how-to-expose-a-function-returning-a-c-object-to-python-without-copying-the-ob/33677627#33677627
cdef extern from "<utility>":
    vector[cplfunction]&& move(vector[cplfunction]&&) #

cdef class Pycplfunctionvec:
    cdef cplfunctionvec thisptr


#https://stackoverflow.com/questions/18260095/cant-override-init-of-class-from-cython-extension
# Trick https://stackoverflow.com/questions/13201886/cython-and-constructors-of-classes
    #def __cinit__(self, vector[double] S1, vector[double] B1, vector[double] f0):
    def __cinit__(self, S1=None,B1=None,f0=None,S2=None,B2=None):
        if S2 is not None and B2 is not None:
            self.thisptr = cplfunctionvec(S1,S2,B1,B2,f0)
        else:
            if S1 is not None and B1 is not None and f0 is not None:
                self.thisptr = cplfunctionvec(S1,B1,f0)
            else:
                self.thisptr = cplfunctionvec()

    #
    # def __dealloc__(self):
    #     del self.thisptr

    cdef copy(self, cplfunctionvec &other):
        self.thisptr = cplfunctionvec(other)

    def size(self):
        return(self.thisptr.size())

    def __repr__(self):
        return "<cplfunctionvec: MycplfunctionList_={}>".format(self.MycplfunctionList_)

#    property MycplfunctionList_:
#        def __get__(self): return self.thisptr.MycplfunctionList_
#        def __set__(self, vector[cplfunction] MycplfunctionList_): self.thisptr.MycplfunctionList_ = MycplfunctionList_


    def Maxf_1Breaks_withO(self,vector[double] S1, vector[double] B1, vector[double] f0):
        self.thisptr.Maxf_1Breaks_withO( S1,  B1, f0)

    def Maxf_2Breaks_withO(self,vector[double] S1,vector[double] S2, vector[double] B1, vector[double] B2,vector[double] f0):
        self.thisptr.Maxf_2Breaks_withO( S1, S2, B1,B2, f0)

    def OptimMargInt(self,vector[double]  Pmoins,vector[double]  Pplus,vector[double]  Cmoins,vector[double] Cplus):
        return(self.thisptr.OptimMargInt( Pmoins, Pplus, Cmoins, Cplus))

    def Evalf(self, vector[double] x):
        return(self.thisptr.Evalf(x))

    def EvalDeltafMoins(self,vector[double] x):
        return(self.thisptr.EvalDeltafMoins(x))

    def EvalDeltafPlus(self,vector[double] x):
        return(self.thisptr.EvalDeltafPlus(x))

    def Max_(self, Pycplfunctionvec y):
        cdef Pycplfunctionvec Pyresvec
        Pyresvec = Pycplfunctionvec()
        Pyresvec.thisptr.Max_(self.thisptr,y.thisptr)
        return(Pyresvec)

    #def
    def vec_get(self,i):
        cdef Pycplfunction Pyres
        Pyres = Pycplfunction([0.],[0.],0.)
        Pyres.copy(self.thisptr.vec_get(i))
        return(Pyres)

    def MaxPycplfunctionvec(Pycplfunctionvec x, Pycplfunctionvec y):
        cdef Pycplfunctionvec Pyresvec
        Pyresvec = Pycplfunctionvec()
        Pyresvec.thisptr.Max_(x.thisptr,y.thisptr)
        return Pyresvec




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat May 23 14:49:39 2020

@author: robin.girard
"""
import dynprogstorage
#from dynprogstorage import linearfunc
#cplfunction_obj = dynprogstorage.Pycplfunction(x0, y0, x1, y1)


#### test Pycplfunction
A=dynprogstorage.Pycplfunction([-1.,1.],[-10.,0.],0.)
A.getBreakPoints()
A.evalf(1)
A.evalf(0)
A.AddSimple(1.,2.,3.,4.)
A.getBreakPoints()

import math
B=dynprogstorage.Pycplfunction([-1,1],[-math.inf,0],0.)
B.getBreakPoints()

#### test Pycplfunctionvec
C= dynprogstorage.Pycplfunctionvec([-1.,1.],[-10.,0.],[0,0])
C.size()
C.Maxf_1Breaks_withO([-1.,3.],[-12.,2.],[0,0])
E= dynprogstorage.Pycplfunctionvec([-2.,2.],[-12.,2.],[0,0])
E=E.Max_(D,C)

import dynprogstorage
from numpy import random
import math


#### cas avec S1, B1, f0
n=100
S1=random.uniform(low=1,high=10,size=n).tolist()
E= dynprogstorage.Pycplfunctionvec(S1,[-math.inf]*n,[0]*n)
PP=1; CC=5*PP;
xEtoile=E.OptimMargInt([-PP]*n,[PP]*n,[0]*n,[CC]*n)
xEtoile

#### cas avec 2 breakpoints S1, B1, f0 , S2, B2
n=100
S1=random.uniform(low=1,high=10,size=n)
S2=S1+2
E= dynprogstorage.Pycplfunctionvec(S1.tolist(),[-math.inf]*n,[0]*n,#S1, B1, f0
                                   S2.tolist(),[0]*n) #S2, B2
PP=1; CC=5*PP;
xEtoile=E.OptimMargInt([-PP]*n,[PP]*n,[0]*n,[CC]*n)

from dynprogstorage import wrappers
from numpy import random

### exemples simples d'utilisation de l'outil de programmation dynamique
###### storage operation example
nbTime=250
Prices=random.uniform(1, 1000, nbTime)
p_max=1.;  c_max=10.*p_max;

CostFunction= wrappers.GenCostFunctionFromMarketPrices(Prices)
## x_i>0 : on stocke (on consomme du réseau)
## x_i<0 : on produit
### --> phi_i(x_i) est donc un coût Achat - vente que l'on veut minimiser
res=CostFunction.OptimMargInt([-p_max]*nbTime,[p_max]*nbTime,[0]*nbTime,[c_max]*nbTime)
## min sum_i phi_i(x_i)
## -p_max <= x_i <=  p_max forall i
## 0 <= sum_j=1^ix_j <= C_max  forall i


#plt.plot(res)
#plt.show()


# #!/usr/bin/env python3
# # -*- coding: utf-8 -*-
# """
# Created on Sat May 30 15:46:28 2020
#
# @author: robin.girard
# """
#
from dynprogstorage.Wrapper_dynprogstorage import Pycplfunctionvec
#from dynprogstorage import linearfunc
import numpy
import math


def pmin(x,y):
    n=len(x)
    TMP= numpy.concatenate((numpy.array(x),numpy.array(y))).reshape(2,n)
    return(numpy.amin(TMP,axis=0).tolist())

def pmax(x,y):
    n=len(x)
    TMP= numpy.concatenate((numpy.array(x),numpy.array(y))).reshape(2,n)
    return(numpy.amax(TMP,axis=0).tolist())

def GenCostFunctionFromMarketPrices(Prices,r_in=1,r_out=1,valueAtZero=0):
    #import dynprogstorage
    Prices=numpy.array(Prices)
    n=len(Prices)

    if ((type(valueAtZero)==int)|(type(valueAtZero)==float)) : valueAtZero=[valueAtZero]*n

    if (type(valueAtZero)==list) : valueAtZero = numpy.array(valueAtZero)

    if (r_in*r_out==1):
        Res= Pycplfunctionvec(Prices.tolist(),[-math.inf]*n,valueAtZero.tolist())
    else:
        Prices_in = Prices * r_out
        Prices_out = Prices / r_in
        Res= Pycplfunctionvec(Prices_in.tolist(),[-math.inf]*n,valueAtZero.tolist(),Prices_out.tolist(),[0]*n)

    return Res;

def GenCostFunctionFromMarketPrices_dict(Prices,r_in=1,r_out=1,valueAtZero=0):
    #import dynprogstorage
    Prices=numpy.array(Prices)
    n=len(Prices)

    if ((type(valueAtZero)==int)|(type(valueAtZero)==float)) : valueAtZero=[valueAtZero]*n

    if (type(valueAtZero)==list) : valueAtZero = numpy.array(valueAtZero)


    if (r_in*r_out==1):
        Res= {"S1" : Prices.tolist(),"B1" : [-math.inf]*n,"f0" :valueAtZero.tolist()}
    else:
        Prices_in = Prices * r_out
        Prices_out = Prices / r_in
        Res= {"S1" : Prices_in.tolist(),"B1" : [-math.inf]*n, "f0" : valueAtZero.tolist(), "S2" : Prices_out.tolist(),"B2" : [0]*n}

    return Res;




#https://packaging.python.org/guides/distributing-packages-using-setuptools/

import os
from setuptools import setup, find_packages
#from distutils.core import setup
#from distutils.core import Extension
#from distutils.extension import Extension
#from setuptools import setup
try:
    from Cython.Build import cythonize
    from Cython.Distutils import build_ext
except ImportError:
    cythonize = None



from distutils.core import setup
from distutils.extension import Extension



# https://cython.readthedocs.io/en/latest/src/userguide/source_files_and_compilation.html#distributing-cython-modules
def no_cythonize(extensions, **_ignore):
    for extension in extensions:
        sources = []
        for sfile in extension.sources:
            path, ext = os.path.splitext(sfile)
            if ext in (".pyx", ".py"):
                if extension.language == "c++":
                    ext = ".cpp"
                else:
                    ext = ".c"
                sfile = path + ext
            sources.append(sfile)
        extension.sources[:] = sources
    return extensions


indlude_dirs=['']

#indlude_dirs=['C:\\Program Files (x86)\\Windows Kits\\10\\Include\\10.0.18362.0\\ucrt',
              #'C:\\Microsoft Visual Studio\\2019\\Community\\VC\\Tools\\MSVC\\14.27.29110\\include',
#              'C:\\Program Files (x86)\\Windows Kits\\10\\Include\\10.0.18362.0\\shared',
              # 'C:\\Microsoft Visual Studio\\2019\\Community\\VC\\Tools\\MSVC\\14.27.29110\\lib\\onecore\\x64',
#              'C:\\Program Files (x86)\\Microsoft Visual Studio 10.0\\VC\\lib\\amd64']
              # 'C:\\Program Files (x86)\\Windows Kits\\10\\bin\\10.0.18362.0\\x64']

dynprogstorage_ext = [Extension('dynprogstorage.Wrapper_dynprogstorage',
    sources=['dynprogstorage/Wrapper_dynprogstorage.pyx',
            #'dynprogstorage/Wrapper_dynprogstorage.cpp',
            #'dynprogstorage/cplfunction.cpp',
            #'dynprogstorage/cplfunction.hpp',
             ],

#    include_dirs=indlude_dirs,  # put include paths here
#    library_dirs=['C:\\Program Files (x86)\\Microsoft Visual Studio 10.0\\VC\\lib\\amd64',
#                  'C:\\Microsoft Visual Studio\\2019\\Community\\VC\\Tools\\MSVC\\14.27.29110\\lib\\onecore\\x64',
                  #,
#                  'C:\\Program Files (x86)\\Windows Kits\\10\\Lib\\10.0.18362.0\\um\\x64',
#                  'C:\\Program Files (x86)\\Windows Kits\\10\\Lib\\10.0.18362.0\\ucrt\\x64'],  # usually need your Windows SDK stuff here
    language='c++',
    #libraries=['boost_python38'],
    #extra_compile_args=['-std=c++11']
)]

#extensions = [
#    Extension("cypack.utils", ["src/cypack/utils.pyx"]),
#    Extension("cypack.answer", ["src/cypack/answer.pyx"]),
#    Extension("cypack.fibonacci", ["src/cypack/fibonacci.pyx"]),
#    Extension(
#        "cypack.sub.wrong",
#        ["src/cypack/sub/wrong.pyx", "src/cypack/sub/helper.c"]
#    ),
#]

CYTHONIZE = bool(int(os.getenv("CYTHONIZE", 0))) and cythonize is not None

if CYTHONIZE:
    compiler_directives = {"language_level": 3, "embedsignature": True}
    extensions = cythonize(dynprogstorage_ext, compiler_directives=compiler_directives)
else:
    extensions = no_cythonize(dynprogstorage_ext)

#set INCLUDE=C:\Program Files (x86)\Windows Kits\10\Include\10.0.17763.0\ucrt
#set INCLUDE=C:\Program Files (x86)\Windows Kits\10\Include\10.0.17763.0\ucrt;C:\Program Files (x86)\Windows Kits\10\Include\10.0.17763.0\shared;C:\Program Files (x86)\Microsoft Visual Studio 10.0\VC\lib\amd64;C:\Program Files (x86)\Windows Kits\10\bin\10.0.17763.0\x64
#set LIB=C:\Program Files (x86)\Microsoft Visual Studio 10.0\VC\lib\amd64;C:\Program Files (x86)\Windows Kits\10\Lib\10.0.17763.0\um\x64;C:\Program Files (x86)\Windows Kits\10\Lib\10.0.17763.0\ucrt\x64;
#libboost_python38.dylib
#C:\Users\robin.girard\AppData\Local\Programs\Python\Python38-32\Scripts\;C:\Users\robin.girard\AppData\Local\Programs\Python\Python38-32\;%PyCharm%;C:\Program Files (x86)\Windows Kits\10\Include
#set INCLUDE=C:\Program Files (x86)\Windows Kits\10\Include\10.0.18362.0\ucrt;C:\Microsoft Visual Studio\2019\Community\VC\Tools\MSVC\14.27.29110\include;C:\Program Files (x86)\Windows Kits\10\Include\10.0.18362.0\shared;C:\Microsoft Visual Studio\2019\Community\VC\Tools\MSVC\14.27.29110\lib\onecore\x64;C:\Program Files (x86)\Microsoft Visual Studio 10.0\VC\lib\amd64;C:\Program Files (x86)\Windows Kits\10\bin\10.0.18362.0\x64

#set LIB=C:\Microsoft Visual Studio\2019\Community\VC\Tools\MSVC\14.27.29110\lib\onecore\x64;C:\Program Files (x86)\Microsoft Visual Studio 10.0\VC\lib\amd64;C:\Program Files (x86)\Windows Kits\10\Lib\10.0.18362.0\um\x64;C:\Program Files (x86)\Windows Kits\10\Lib\10.0.18362.0\um\x64;C:\Program Files (x86)\Windows Kits\10\Lib\10.0.18362.0\ucrt\x64
setup(
    cmdclass={'build_ext': build_ext},
 #   setup_args={'script_args': ["--compiler=msvc"]},
    name='dynprogstorage',
    version='0.1.10',
    ext_modules=extensions,
    #long_description=readme,
    #long_description_content_type='text/markdown',
    author='Robin Girard',
    author_email='robin.girard@mines-paristech.fr',license=license,
    include_dirs = indlude_dirs,
    packages=find_packages(),

)

